#!/usr/local/bin/perl
#
# Perl Script to generate an R Fisher's Exact Test script to carry out an analysis of each of the clusters from the POSOC full gene list analysis. This compares a supplied list of differentially expressed genes with the 'all gene' clusters to statistically analyse whether differentially expressed genes are significantly over- or under-represented in the Cluster. 
# This script requires three input files, a list of differentially expressed genes or Affy IDs (one per line), and a POSOC output which should be for 'all genes' in the species concerned. The way in which the genes are represented in the latter should be identical to the former -- i.e. if the list of differentially expressed genes contains Affy IDs, the POSOC output should also contain Affy IDs. The filename of the transfile output by affyfile_converter.pl should be included as the third command line parameter.
# Two other optional command line parameters can be included. The first of these is the type of Fisher's test desired. Fisher's tests can be one-tailed or two-tailed. If a one tailed test is required, enter 'greater' or 'less' (without quotation marks) as the fourth command line parameter. Otherwise enter anything else (no whitespace permitted) to carry out a two tailed test. A fifth command line parameter is the minimum number of differentially expressed genes that may be present in a Cluster for a Fisher's Exact Test to be carried out on it. This is used to reduce the problem of multiple testing. The minimum number of genes should be entered as a figure.
# If a minimum number of genes is desired, information must be to indicate the type of Fisher's test required. If this is not provided, the requested minimum will be overlooked by the program.
# Ian Grieve
# 21/07/05

use strict;
use statistics::R;

our $not_annotated_count = 0;

our $fisher_inputfile_de = "";
our $posoc_clusters_all = "";
our $genes_GO_terms = "";
my $de_gene_id = "";
my $tailed = "";
my $annotated_genes_total = "";
my $min_de_genes = 0;

if ($#ARGV < 2)
{
    die print "Please supply the command line arguments required\n Three command line arguments should be supplied.";
}

else
{
    $fisher_inputfile_de = $ARGV[0];
    $posoc_clusters_all = $ARGV[1];
    $genes_GO_terms = $ARGV[2]; 
    $tailed = $ARGV[3];
    $min_de_genes = $ARGV[4];
}

sub find_de_noannotations

# This subroutine outputs the number of the differentially expressed genes which have no annotation. This is required for the third cell of the Fisher's Test 2x2 table later.
{
    close (DE_GENELIST);
    close (POSOC_CLUSTERS);
    my $if_present_annotated = $genes_GO_terms;
    my $annotated_count = 0;
    my $total_count_debug = 0;
    my $de_genes = $fisher_inputfile_de;
    my $find_de_noannotations_debug = 'find_de_noannotations.txt';
    my $find_de_annotations_debug = 'find_de_annotations.txt';
    open (DE_GENES, $de_genes) || die print "Cannot open $de_genes in Fisher's Test Generator, find_de_noannotations subroutine";
    open (IF_PRESENT_ANNOTATED, $if_present_annotated) || die print "Cannot open $if_present_annotated in Fisher's Test Generator, find_de_noannotations subroutine";
    open (NOANNO_DEBUG, ">$find_de_noannotations_debug") || die print "Cannot open $find_de_noannotations_debug in Fisher's Test Generator, find_de_noannotations subroutine";
    open (ANNO_DEBUG, ">$find_de_annotations_debug") || die print "Cannot open $find_de_annotations_debug in Fisher's Test Generator, find_de_annotations subroutine";

    $/ = "\n";
    my @if_present_annotated = <IF_PRESENT_ANNOTATED>;
    my $if_present_annotated_tosearch = "@if_present_annotated";
    $annotated_genes_total = $#if_present_annotated;
    my @de_genes = <DE_GENES>;
    
    if ($de_genes[0] !~ /_[a|s]t/)
    {
	shift (@de_genes);
    }

    foreach my $de_gene (@de_genes)
{
    chomp $de_gene;
    $de_gene =~ /([^\s]*)\s*/;
    $de_gene_id = $1;
#    print $de_gene_id;
#    print "\n";

    if ($if_present_annotated_tosearch =~ /$de_gene_id/)
    {
	# For each Affy ID in the list of differentially expressed Affy IDs, finds whether it is also found in the list of annotated Affy IDs (i.e. the transfile).
	$annotated_count = $annotated_count + 1;
	print ANNO_DEBUG $de_gene_id;
	print ANNO_DEBUG "\n";
	
    }

    else 
    {
	$not_annotated_count = $not_annotated_count + 1;
	print NOANNO_DEBUG $de_gene_id;
	print NOANNO_DEBUG "\n";
    }
}
    $total_count_debug = $annotated_count + $not_annotated_count;
    print "Annotated: ";
    print $annotated_count;
    print "\n";
    print "Not Annotated: ";
    print $not_annotated_count;
    print "\n";
    print "Total: ";
    print $total_count_debug;
    print "\n";
}

find_de_noannotations;

close (NOANNO_DEBUG);
close (ANNO_DEBUG);

unlink ('find_de_annotations.txt') || die print "Fisher Test Generator cannot unlink find_de_annotations.txt";
unlink ('find_de_noannotations.txt') || die print "Fisher Test Generator cannot unlink find_de_noannotations.txt";

our $de_genes = $fisher_inputfile_de;
our $posoc_all_clusters = $posoc_clusters_all;

our $fisher_gen_debug = 'fisher_gen_debug.txt';
open (FISHERDEBUG, ">$fisher_gen_debug") || die print "Cannot open $fisher_gen_debug in All Genes Fisher's Test Generator";

our $fisher_out = 'fishertests.r';
open (FISHEROUT, ">$fisher_out") || die print "Cannot open $fisher_out in All Genes Fisher's Test Generator";

open (POSOC_CLUSTERS, $posoc_all_clusters) || die print "Cannot open $posoc_all_clusters in All Genes Fisher's Test Generator";

open (DE_GENELIST, $de_genes) || die print "Cannot open $de_genes in All Genes Fisher's Test Generator";

my @de_genelist = <DE_GENELIST>;
my $de_gene_tomatch = "";
my $cluster_name_print = "";
my $cluster_name_discard = "";
my $cluster_de_counter = 0;
my $cluster_nonde_counter = 0;
my $noncluster_nonde_counter = 0;
my $fisher_test_check_value = 0;
my $de_notcluster_counter = 0;
my $de_notcluster_anonly = 0;
my $cluster_number = "";
my $cluster_totalgenes = "";
my $cluster_noncluster_totalgenes = 0;
my $de_totalgenes = "";
my $fisher_test_name = "";
my $fisher_test_name_head = "Cluster";
my $full_totalgenes_counter = 0;

my @posoc_clusters_tosplit = <POSOC_CLUSTERS>;
my $posoc_clusters_tosplit = "@posoc_clusters_tosplit";
my @posoc_clusters = split /-{45}/, $posoc_clusters_tosplit;

my $posoc_cluster_genes = $posoc_clusters[2];
my @posoc_cluster_genes = split /Cluster\s/, $posoc_cluster_genes;

my $total_genes_clusters_info = @posoc_cluster_genes[0];
$total_genes_clusters_info =~ /(\d+) genes/;
my $clusters_genes_total = $1;


shift @posoc_cluster_genes;

# This section of the script finds the differentially expressed genes for each Cluster and counts them. It also outputs the DE Affy IDs for each cluster into a temporary file (fisher_gen_debug.txt).

if ($de_genelist[0] !~ /_[a|s]t/)
    {
	shift (@de_genelist);
    } 

foreach my $cluster (@posoc_cluster_genes)
{
    $cluster =~ /^(\d+)\((\d+) genes\): \*?GO:\d+ ([^=]*)=/;
    $cluster_number = $1;
    $cluster_totalgenes = $2;
    ($cluster_name_print, $cluster_name_discard) = split / score/, $3;
    
    print FISHERDEBUG "Cluster ";
    print FISHERDEBUG $cluster_number;
    print FISHERDEBUG "\t";
    print FISHERDEBUG $cluster_name_print;
    print FISHERDEBUG "\n";
    $cluster_de_counter = 0;
    $de_notcluster_counter = 0;
    $full_totalgenes_counter = $full_totalgenes_counter + $cluster_totalgenes;

    foreach my $de_gene (@de_genelist)
    {
	$de_gene =~ /([^\s]*[a|s]t)\s*/;
	$de_gene_tomatch = $1;
	if ($cluster =~ /$de_gene_tomatch/)
	{
	    print FISHERDEBUG $de_gene_tomatch;
	    print FISHERDEBUG "\n";
	    $cluster_de_counter = $cluster_de_counter + 1;
	    # Cluster_de_counter counts the number of differentially expressed genes in each cluster
	}
	
	else 
	{
	    $de_notcluster_counter = $de_notcluster_counter + 1
            # DE_notcluster_counter counts those differentially expressed genes that are not in the cluster being tested.
	}
    
    }

    if ($cluster_de_counter < $min_de_genes)
# This if statement is used if a minimum number of DE genes to generate a p-value is selected on the command line.
    {
	next;
    }
    
    else
    {
	$de_notcluster_anonly = $de_notcluster_counter - $not_annotated_count;
	$de_totalgenes = $cluster_de_counter + $de_notcluster_counter;
	$cluster_nonde_counter = $cluster_totalgenes - $cluster_de_counter;
	$noncluster_nonde_counter = $clusters_genes_total - $cluster_de_counter - $cluster_nonde_counter - $de_notcluster_anonly;
	$fisher_test_name = $fisher_test_name_head.$cluster_number;
	
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Total Count of Differentially Expressed Genes in Cluster ";
	print FISHERDEBUG $cluster_number;
	print FISHERDEBUG ": ";
	print FISHERDEBUG $cluster_de_counter;
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Total Count of Non-Differentially Expressed Genes in Cluster ";
	print FISHERDEBUG $cluster_number;
	print FISHERDEBUG ": ";
	print FISHERDEBUG $cluster_nonde_counter;
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Total Count of Genes in Cluster ";
	print FISHERDEBUG $cluster_number;
	print FISHERDEBUG ": ";
	print FISHERDEBUG $cluster_totalgenes;
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Total Count of Differentially Expressed Genes not in Cluster ";
	print FISHERDEBUG $cluster_number;
	print FISHERDEBUG ": ";
	print FISHERDEBUG $de_notcluster_anonly;
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Total No. of Differentially Expressed Genes: ";
	print FISHERDEBUG $noncluster_nonde_counter;
	print FISHERDEBUG "\n";
	print FISHERDEBUG "Fisher Values: ";
	print FISHERDEBUG $cluster_de_counter;
	print FISHERDEBUG ", ";
	print FISHERDEBUG $cluster_nonde_counter;
	print FISHERDEBUG ", ";
	print FISHERDEBUG $de_notcluster_anonly;
	print FISHERDEBUG ", ";
	print FISHERDEBUG $noncluster_nonde_counter; 
	print FISHERDEBUG "\n\n\n";

	print FISHEROUT "## Fisher's Exact Test for Cluster ";
	print FISHEROUT $cluster_number;
	print FISHEROUT " \( ";
	print FISHEROUT $cluster_name_print;
	print FISHEROUT "\)";
	print FISHEROUT "\n\n";
	print FISHEROUT $fisher_test_name;
	print FISHEROUT " <-";
	print FISHEROUT "\n";
	print FISHEROUT "matrix\(c\(";
	print FISHEROUT $cluster_de_counter;
	print FISHEROUT ", ";
	print FISHEROUT $cluster_nonde_counter;
	print FISHEROUT ", ";
	print FISHEROUT $de_notcluster_anonly;
	print FISHEROUT ", ";
	print FISHEROUT $noncluster_nonde_counter;
	print FISHEROUT "\),";
	print FISHEROUT "\n";
	print FISHEROUT "nr = 2,";
	print FISHEROUT "\n";
	print FISHEROUT "dimnames = list\(Annotated = c\(\"DE\", \"Non-DE\"\),";
	print FISHEROUT "\n";
	print FISHEROUT "NotAnnotated = c\(\"DE\", \"Non-DE\"\)\)\)";
	print FISHEROUT "\n";
	print FISHEROUT "fisher.test\(";
	print FISHEROUT $fisher_test_name;
#    print FISHEROUT "\)";
	
    if ($tailed =~ /greater/)
    {
	print FISHEROUT ", alternative = \"greater\"\)\n\n";
    }

    elsif ($tailed =~ /less/)
    {
	print FISHEROUT ", alternative = \"less\"\)\n\n";
    }

    else {    print FISHEROUT "\)\n\n\n";  }
}

}

close (FISHERDEBUG);

unlink ('fisher_gen_debug.txt') || die print "Fisher Test Generator cannot unlink fisher_gen_debug.txt";

close (FISHEROUT);
close (DE_GENES);
close (DE_GENELIST);

my $R_send_fisherscript = Statistics::R->new();
$R_send_fisherscript->startR;





